# Contributing to the Coscine Python SDK

## General

The Coscine Python SDK is open source software and not developed by
the Coscine team. Instead it is developed by the userbase of Coscine,
which is researchers, data stewards and project managers.  
If you belong to that userbase and would like to improve this package,
you are more than welcome to contribute.  

## Contribution Workflow

The general flow of contributing to this repository is:

1. Create a fork of the repository and make your changes in your fork. In case you are not an RWTH member you may not have full access to the RWTH Gitlab and may not be able to fork projects. If you would like to participate, you need to be added to the project. Please contact [the community team](mailto:coscine-technical-adaptation@itc.rwth-aachen.de) and state your intention of contributing to the project. They will be able to grant you developer rights, which enable you to create merge requests inside the repository.
2. When you’re ready, create a new merge request.
3. Ensure you provide complete and accurate information in the merge
   request’s description
4. Once you’re ready, mark your MR as ready for review.
5. The Maintainers of this package will take a look at your contribution
   and incorporate the changes into the repository, if no fatal flaws
   have been detected.

## Code convention & Style guidelines

- limit the column size to 80 columns for each line
	- Links and other indivisible elements are exempt from this rule
	- When breaking a line in python, use \
	- Set tabsize to 4
- Use line-feed LF
- Set encoding to UTF-8
- Use DOCstrings for modules, classes, methods and functions
	- Use [numpy DOCstring format](https://numpydoc.readthedocs.io/en/latest/format.html)
- Write self descriptive code
	- Variable and function names should indicate what's going on
	- Code should make comments unncessecary (except in rare complicated cases)

Stick to the following template for sourcecode files:
```python
###############################################################################
# Coscine Python SDK
# Copyright (c) 2018-2022 RWTH Aachen University
# #############################################################################
# Coscine, short for Collaborative Scientific Integration Environment is
# a platform for research data management (RDM).
# For more information on Coscine visit https://www.coscine.de/.
#
# Please note that this python module is open source software primarily
# developed and maintained by the scientific community. It is not
# an official service that RWTH Aachen provides support for.
###############################################################################

###############################################################################
# File description
###############################################################################

"""
TODO
"""

###############################################################################
# Dependencies
###############################################################################

from __future__ import annotations

###############################################################################
# Global variables
###############################################################################

###############################################################################
# Class
###############################################################################

###############################################################################
```

- Seperating elements with 80 column comments makes it easier to scroll
  between elements such as functions as their boundaries become apparent
- Seperating sourcecode with different purposes by big 80 column comments
  divides the source file into different sections and gives the file
  a structure, which can be searched through by using a search string
  such as '# section'.

---------------------

## Deployment

(Re-)Build the documentation with  
```bash
cd ./coscine-python-sdk
py -m pip install pdoc3
py -m pdoc --html --force --template-dir src/docs/template -o ./public ./src/coscine
```  

## Maintainers

To deploy code to PyPi and essentially publish a new version of the Python
SDK, create a new Tag in the GitLab-repository with the new version 'vX.X.X'.
Monitor the GitLab-Ci pipeline and check whether everything went ok.
